@@ -3,7 +3,7 @@
 NULL
 
 
-#####.set_labels#####
+####.set_labels#####
 setMethod(".set_labels", signature(x="familiarCollection"),
           function(x, old_label, new_label, new_order, upd_slot){
 
@@ -24,7 +24,6 @@ setMethod(".set_labels", signature(x="familiarCollection"),
               label_table <- slot(x, upd_slot)
             }
             
-            
             ##### Label update ##################################################
             if(!is.null(new_label)){
               
@@ -83,19 +82,7 @@ setMethod(".set_labels", signature(x="familiarCollection"),
             }
             
             ##### Label order update ##################################################
-            if(is.null(new_order)){
-              # If new_order is not provided explicitly, we can still update the order if and only if new_label is as long as the table
-              if(!is.null(new_label)){
-                if(length(new_label) == nrow(label_table)){
-                  
-                  # Find the order by matching the label column in the label table with the provided new labels
-                  new_label_order <- match(label_table$label, new_label)
-                  
-                  # Update the label order
-                  label_table[, "label_order":=new_label_order]
-                }
-              }
-            } else {
+            if(!is.null(new_order)){
               # Update label order based on new_order
 
               # Check that new_order has the same length as the label_table
@@ -157,7 +144,7 @@ setMethod(".construct_label_table", signature(x="familiarCollection"),
               data <- unique(c(slot(x, "required_features"), x@fs_vimp$vimp_table$name))
               
             } else if(upd_slot == "km_group_labels") {
-              data <- unique(c("low", "moderate", "high", unlist(sapply(names(x@km_data), function(curr_method)(levels(x@km_data[[curr_method]]$data$risk_group))))))
+              data <- unique(c("low", "moderate", "high", unlist(lapply(x@km_data, function(x)(levels(x@data$risk_group))))))
 
             } else if(upd_slot == "class_labels") {
               data <- get_outcome_class_levels(x)
@@ -207,30 +194,36 @@ setMethod(".get_labels", signature(x="familiarCollection"),
 
 
 #####set_data_set_names#####
+
 #' @title Name datasets for plotting and export
-#'  
-#' @description Tabular exports and figures created from a familiarCollection object can be customised by setting data labels.
-#' 
-#' @details Labels convert internal naming of data sets to the requested label at export or when plotting. Currently assigned labels
-#'  can be found using the \code{get_data_set_names} method.
+#'
+#' @description Tabular exports and figures created from a familiarCollection
+#'   object can be customised by setting data labels.
+#'
+#' @details Labels convert internal naming of data sets to the requested label
+#'   at export or when plotting. Currently assigned labels can be found using
+#'   the \code{get_data_set_names} method.
 #'
 #' @param x A familiarCollection object.
 #' @param old (optional) Set of old labels to replace.
-#' @param new Set of replacement labels. The number of replacement labels should be equal to the number of provided old labels or the full number of labels.
-#'   If a subset of labels is to be replaced, both \code{old} and \code{new} should be provided.
-#' @param order (optional) Ordered set of replacement labels. This is used to provide the order 
-#'   in which the labels should be placed, which affects e.g. levels in a plot. If the ordering is not explicitly provided,
-#'   and \code{new} matches the total number of labels in length, the ordering of \code{new} is used to order the table.
+#' @param new Set of replacement labels. The number of replacement labels should
+#'   be equal to the number of provided old labels or the full number of labels.
+#'   If a subset of labels is to be replaced, both \code{old} and \code{new}
+#'   should be provided.
+#' @param order (optional) Ordered set of replacement labels. This is used to
+#'   provide the order in which the labels should be placed, which affects e.g.
+#'   levels in a plot. If the ordering is not explicitly provided, the old
+#'   ordering is used.
 #'
 #' @return A familiarCollection object with custom names for the data sets.
 #' @export
 #' @aliases set_data_set_names
-#' @seealso
-#' * \linkS4class{familiarCollection} for information concerning the familiarCollection class.
-#' * \code{\link{get_data_set_names}} for obtaining currently assigned labels.
+#' @seealso * \linkS4class{familiarCollection} for information concerning the
+#' familiarCollection class. * \code{\link{get_data_set_names}} for obtaining
+#' currently assigned labels.
 #' @md
 setMethod("set_data_set_names", signature(x="familiarCollection"),
-          function(x, old=NULL, new, order=NULL){
+          function(x, old=NULL, new=NULL, order=NULL){
             x <- .set_labels(x=x, old_label=old, new_label=new, new_order=order, upd_slot="data_set_labels")
             return(x)
           })
@@ -238,24 +231,28 @@ setMethod("set_data_set_names", signature(x="familiarCollection"),
 
 
 #####set_learner_names#####
+
 #' @title Rename learners for plotting and export
-#'  
-#' @description Tabular exports and figures created from a familiarCollection object can be customised by providing names for the learners.
-#' 
-#' @details Labels convert the internal naming for learners to the requested label at export or when plotting. This enables the use of
-#'  more specific naming, e.g. changing \code{random_forest_rfsrc} to \code{Random Forest}. Currently assigned labels
-#'  can be found using the \code{get_learner_names} method.
+#'
+#' @description Tabular exports and figures created from a familiarCollection
+#'   object can be customised by providing names for the learners.
+#'
+#' @details Labels convert the internal naming for learners to the requested
+#'   label at export or when plotting. This enables the use of more specific
+#'   naming, e.g. changing \code{random_forest_rfsrc} to \code{Random Forest}.
+#'   Currently assigned labels can be found using the \code{get_learner_names}
+#'   method.
 #'
 #' @inheritParams set_data_set_names,familiarCollection-method
 #' @return A familiarCollection object with custom labels for the learners.
 #' @export
 #' @aliases set_learner_names
-#' @seealso 
-#' * \linkS4class{familiarCollection} for information concerning the familiarCollection class.
-#' * \code{\link{get_learner_names}} for obtaining currently assigned labels.
+#' @seealso * \linkS4class{familiarCollection} for information concerning the
+#' familiarCollection class. * \code{\link{get_learner_names}} for obtaining
+#' currently assigned labels.
 #' @md
 setMethod("set_learner_names", signature(x="familiarCollection"),
-          function(x, old=NULL, new, order=NULL){
+          function(x, old=NULL, new=NULL, order=NULL){
             x <- .set_labels(x=x, old_label=old, new_label=new, new_order=order, upd_slot="learner_labels")
             return(x)
           })
@@ -263,24 +260,29 @@ setMethod("set_learner_names", signature(x="familiarCollection"),
 
 
 #####set_fs_method_names#####
+
 #' @title Rename feature selection methods for plotting and export
-#'  
-#' @description Tabular exports and figures created from a familiarCollection object can be customised by providing names for the feature selection methods.
-#' 
-#' @details Labels convert the internal naming for feature selection methods to the requested label at export or when plotting. This enables the use of
-#'   more specific naming, e.g. changing \code{mim} to \code{Mutual Information Maximisation}. Currently assigned labels
-#'  can be found using the \code{get_fs_method_names} method.
+#'
+#' @description Tabular exports and figures created from a familiarCollection
+#'   object can be customised by providing names for the feature selection
+#'   methods.
+#'
+#' @details Labels convert the internal naming for feature selection methods to
+#'   the requested label at export or when plotting. This enables the use of
+#'   more specific naming, e.g. changing \code{mim} to \code{Mutual Information
+#'   Maximisation}. Currently assigned labels can be found using the
+#'   \code{get_fs_method_names} method.
 #'
 #' @inheritParams set_data_set_names,familiarCollection-method
 #' @return A familiarCollection object with updated labels.
 #' @export
 #' @aliases set_fs_method_names
-#' @seealso 
-#' * \linkS4class{familiarCollection} for information concerning the familiarCollection class.
-#' * \code{\link{get_fs_method_names}} for obtaining currently assigned labels.
+#' @seealso * \linkS4class{familiarCollection} for information concerning the
+#' familiarCollection class. * \code{\link{get_fs_method_names}} for obtaining
+#' currently assigned labels.
 #' @md
 setMethod("set_fs_method_names", signature(x="familiarCollection"),
-          function(x, old=NULL, new, order=NULL){
+          function(x, old=NULL, new=NULL, order=NULL){
             x  <- .set_labels(x=x, old_label=old, new_label=new, new_order=order, upd_slot="fs_method_labels")
             return(x)
           })
@@ -288,24 +290,27 @@ setMethod("set_fs_method_names", signature(x="familiarCollection"),
 
 
 #####set_feature_names#####
+
 #' @title Rename features for plotting and export
-#'  
-#' @description Tabular exports and figures created from a familiarCollection object can be customised by providing names for features.
-#' 
-#' @details Labels convert the internal naming for features to the requested label at export or when plotting. This enables customisation
-#'   without redoing the analysis with renamed input data. Currently assigned labels
-#'  can be found using the \code{get_feature_names} method.
+#'
+#' @description Tabular exports and figures created from a familiarCollection
+#'   object can be customised by providing names for features.
+#'
+#' @details Labels convert the internal naming for features to the requested
+#'   label at export or when plotting. This enables customisation without
+#'   redoing the analysis with renamed input data. Currently assigned labels can
+#'   be found using the \code{get_feature_names} method.
 #'
 #' @inheritParams set_data_set_names,familiarCollection-method
 #' @return A familiarCollection object with updated labels.
 #' @export
 #' @aliases set_feature_names
-#' @seealso
-#' * \linkS4class{familiarCollection} for information concerning the familiarCollection class.
-#' * \code{\link{get_feature_names}} for obtaining currently assigned feature names.
+#' @seealso * \linkS4class{familiarCollection} for information concerning the
+#' familiarCollection class. * \code{\link{get_feature_names}} for obtaining
+#' currently assigned feature names.
 #' @md
 setMethod("set_feature_names", signature(x="familiarCollection"),
-          function(x, old=NULL, new, order=NULL){
+          function(x, old=NULL, new=NULL, order=NULL){
             x <- .set_labels(x=x, old_label=old, new_label=new, new_order=order, upd_slot="feature_labels")
             return(x)
           })
@@ -313,23 +318,28 @@ setMethod("set_feature_names", signature(x="familiarCollection"),
 
 
 #####set_risk_group_names#####
+
 #' @title Rename risk groups for plotting and export
-#'  
-#' @description Tabular exports and figures created from a familiarCollection object can be customised by providing names for risk groups in survival analysis.
-#' 
-#' @details Labels convert the internal naming for risk groups to the requested label at export or when plotting. This enables customisation of risk group names.
-#'   Currently assigned labels can be found using the \code{get_risk_group_names} method.
+#'
+#' @description Tabular exports and figures created from a familiarCollection
+#'   object can be customised by providing names for risk groups in survival
+#'   analysis.
+#'
+#' @details Labels convert the internal naming for risk groups to the requested
+#'   label at export or when plotting. This enables customisation of risk group
+#'   names. Currently assigned labels can be found using the
+#'   \code{get_risk_group_names} method.
 #'
 #' @inheritParams set_data_set_names,familiarCollection-method
 #' @return A familiarCollection object with updated labels.
 #' @export
 #' @aliases set_risk_group_names
-#' @seealso
-#' * \linkS4class{familiarCollection} for information concerning the familiarCollection class.
-#' * \code{\link{get_risk_group_names}} for obtaining currently assigned risk group labels.
+#' @seealso * \linkS4class{familiarCollection} for information concerning the
+#' familiarCollection class. * \code{\link{get_risk_group_names}} for obtaining
+#' currently assigned risk group labels.
 #' @md
 setMethod("set_risk_group_names", signature(x="familiarCollection"),
-          function(x, old=NULL, new, order=NULL){
+          function(x, old=NULL, new=NULL, order=NULL){
             x <- .set_labels(x=x, old_label=old, new_label=new, new_order=order, upd_slot="km_group_labels")
             return(x)
           })
@@ -337,23 +347,27 @@ setMethod("set_risk_group_names", signature(x="familiarCollection"),
 
 
 #####set_class_names#####
+
 #' @title Rename outcome classes for plotting and export
-#'  
-#' @description Tabular exports and figures created from a familiarCollection object can be customised by providing names for outcome classes.
-#' 
-#' @details Labels convert the internal naming for class levels to the requested label at export or when plotting. This enables customisation of class names.
-#'   Currently assigned labels can be found using the \code{get_class_names} method.
+#'
+#' @description Tabular exports and figures created from a familiarCollection
+#'   object can be customised by providing names for outcome classes.
+#'
+#' @details Labels convert the internal naming for class levels to the requested
+#'   label at export or when plotting. This enables customisation of class
+#'   names. Currently assigned labels can be found using the
+#'   \code{get_class_names} method.
 #'
 #' @inheritParams set_data_set_names,familiarCollection-method
 #' @return A familiarCollection object with updated labels.
 #' @export
 #' @aliases set_class_names
-#' @seealso
-#' * \linkS4class{familiarCollection} for information concerning the familiarCollection class.
-#' * \code{\link{get_class_names}} for obtaining currently assigned class names.
+#' @seealso * \linkS4class{familiarCollection} for information concerning the
+#' familiarCollection class. * \code{\link{get_class_names}} for obtaining
+#' currently assigned class names.
 #' @md
 setMethod("set_class_names", signature(x="familiarCollection"),
-          function(x, old=NULL, new, order=NULL){
+          function(x, old=NULL, new=NULL, order=NULL){
             x <- .set_labels(x=x, old_label=old, new_label=new, new_order=order, upd_slot="class_labels")
             return(x)
           })
