@@ -468,27 +468,28 @@ setMethod("plot_univariate_importance", signature(object="familiarCollection"),
               # Save and export
               if(!is.null(dir_path)){
                 
-                # Set subtype.
-                subtype <- plotting.create_subtype(x=x_sub,
-                                                   subtype="univariate",
-                                                   split_by=split_by)
-                
                 # Obtain decent default values for the plot.
-                def_plot_dims <- .determine_univariate_importance_plot_dimensions(x=x_sub,
-                                                                                  facet_by=facet_by,
-                                                                                  facet_wrap_cols=facet_wrap_cols)
-
+                def_plot_dims <- .determine_univariate_importance_plot_dimensions(
+                  x=x_sub,
+                  facet_by=facet_by,
+                  facet_wrap_cols=facet_wrap_cols)
+                
                 # Save to file.
-                do.call(plotting.save_plot_to_file,
-                        args=c(list("plot_obj"=p,
-                                    "object"=object,
-                                    "dir_path"=dir_path,
-                                    "type"="variable_importance",
-                                    "subtype"=subtype,
-                                    "height"=ifelse(is.waive(height), def_plot_dims[1], height),
-                                    "width"=ifelse(is.waive(width), def_plot_dims[2], width),
-                                    "units"=ifelse(is.waive(units), "cm", units)),
-                               list(...)))
+                do.call(
+                  plotting.save_plot_to_file,
+                  args=c(
+                    list(
+                      "plot_obj"=p,
+                      "object"=object,
+                      "dir_path"=dir_path,
+                      "type"="variable_importance",
+                      "subtype"="univariate",
+                      "x"=x_sub,
+                      "split_by"=split_by,
+                      "height"=ifelse(is.waive(height), def_plot_dims[1], height),
+                      "width"=ifelse(is.waive(width), def_plot_dims[2], width),
+                      "units"=ifelse(is.waive(units), "cm", units)),
+                    list(...)))
                 
               } else {
                 # Store as list and export
