@@ -51,9 +51,8 @@ testthat::test_that("Generalised linear model has variable importance", {
   # Expect that the names are the same as that of the features.
   testthat::expect_equal(all(familiar:::get_feature_columns(good_data) %in% vimp_table$name), TRUE)
   
-  # Expect that avg_rooms has rank 1 and residence_before_1940_proportion has rank 2.
-  testthat::expect_equal(vimp_table[rank == 1, ]$name, "avg_rooms")
-  testthat::expect_equal(vimp_table[rank == 2, ]$name, "residence_before_1940_proportion")
+  # Expect that ranks 1 and 2 to be occupied by avg_rooms and lower_status_percentage.
+  testthat::expect_setequal(vimp_table[rank %in% c(1, 2), ]$name, c("avg_rooms", "lower_status_percentage"))
 })
 
 
